# -*- coding: utf-8 -*-
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib import messages
from django.db.models import Max
from django.utils.decorators import method_decorator
from django_tables2 import SingleTableView

from frappe.models import (
    Annee,
    Filiere,
    Matiere,
    Tag,
    Document,
    DocumentFichier,
    Commentaire,
)
from gestion.frappe.forms import (
    AnneeForm,
    FiliereForm,
    MatiereForm,
    TagForm,
    CreateDocumentForm,
    EditDocumentForm,
    CommentaireForm,
)
from gestion.frappe.tables import DocumentTable
from lib.access_auth import permission_needed
from lib.tools import confirmation

from lib.str_tools import dameraulevenshtein


@permission_needed("gerer_frappe")
def index(request):
    return render(request, "gestion/frappe/index.html")


@permission_needed("gerer_frappe")
def annees_liste(request):
    c = {"annees": Annee.objects.filter(ecole=request.ecole).order_by("nom")}
    return render(request, "gestion/frappe/annees_liste.html", c)


@permission_needed("gerer_frappe")
def annees_nouveau(request):
    if request.method == "POST":
        form = AnneeForm(request.POST)
        if form.is_valid():
            annee = form.save(commit=False)
            annee.ecole = request.ecole
            annee.save()
            messages.success(request, "Année créée avec succès.")
            return redirect("gestion:frappe:annees_liste")
    else:
        form = AnneeForm()

    return render(request, "gestion/frappe/annees_nouveau.html", {"form": form})


@permission_needed("gerer_frappe")
def annees_editer(request, id):
    annee = get_object_or_404(Annee.objects.filter(ecole=request.ecole), pk=id)
    if request.method == "POST":
        form = AnneeForm(request.POST, instance=annee)
        if form.is_valid():
            form.save()
            messages.success(request, "Année modifiée avec succès.")
            return redirect("gestion:frappe:annees_liste")
    else:
        form = AnneeForm(instance=annee)

    c = {
        "annee": annee,
        "form": form,
    }
    return render(request, "gestion/frappe/annees_editer.html", c)


@permission_needed("gerer_frappe")
def annees_supprimer(request, id):
    annee = get_object_or_404(Annee.objects.filter(ecole=request.ecole), pk=id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Année supprimée avec succès.")
            annee.delete()

        return redirect("gestion:frappe:annees_liste")

    message = f"Voulez-vous vraiment supprimer l'année « {annee.nom} » ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)


@permission_needed("gerer_frappe")
def filieres_liste(request):
    filieres = Filiere.objects.filter(annee__ecole=request.ecole).order_by("annee__nom", "nom")
    return render(request, "gestion/frappe/filieres_liste.html", {"filieres": filieres})


@permission_needed("gerer_frappe")
def filieres_nouveau(request):
    if request.method == "POST":
        form = FiliereForm(request.POST, ecole=request.ecole)
        if form.is_valid():
            form.save()
            messages.success(request, "Filière créée avec succès.")
            return redirect("gestion:frappe:filieres_liste")
    else:
        form = FiliereForm(ecole=request.ecole)

    return render(request, "gestion/frappe/filieres_nouveau.html", {"form": form})


@permission_needed("gerer_frappe")
def filieres_editer(request, id):
    filiere = get_object_or_404(Filiere.objects.filter(annee__ecole=request.ecole), pk=id)
    if request.method == "POST":
        form = FiliereForm(request.POST, instance=filiere, ecole=request.ecole)
        if form.is_valid():
            form.save()
            messages.success(request, "Filière modifiée avec succès.")
            return redirect("gestion:frappe:filieres_liste")
    else:
        form = FiliereForm(instance=filiere, ecole=request.ecole)

    c = {
        "filiere": filiere,
        "form": form,
    }
    return render(request, "gestion/frappe/filieres_editer.html", c)


@permission_needed("gerer_frappe")
def filieres_supprimer(request, id):
    filiere = get_object_or_404(Filiere.objects.filter(annee__ecole=request.ecole), pk=id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Filière supprimée avec succès.")
            filiere.delete()

        return redirect("gestion:frappe:filieres_liste")

    message = f"Voulez-vous vraiment supprimer la filière « {filiere.nom} » ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)


@permission_needed("gerer_frappe")
def matieres_liste(request):
    c = {
        "matieres": Matiere.objects.filter(filiere__annee__ecole=request.ecole).order_by(
            "filiere__annee__nom", "filiere__nom", "nom"
        )
    }
    return render(request, "gestion/frappe/matieres_liste.html", c)


@permission_needed("gerer_frappe")
def matieres_nouveau(request):
    if request.method == "POST":
        form = MatiereForm(request.POST, ecole=request.ecole)
        if form.is_valid():
            matiere = form.save()
            matieres_proches = [
                x
                for x in Matiere.objects.filter(filiere=matiere.filiere).exclude(id=matiere.id)
                if (dameraulevenshtein(matiere.nom, x.nom) < 3)
            ]
            if len(matieres_proches) > 0:
                return redirect("gestion:frappe:matieres_nouveau_doublon", id=matiere.id)
            else:
                messages.success(request, "Matière créée avec succès.")
                return redirect("gestion:frappe:matieres_liste")

    else:
        form = MatiereForm(ecole=request.ecole)

    return render(request, "gestion/frappe/matieres_nouveau.html", {"form": form})


@permission_needed("gerer_frappe")
def matieres_nouveau_doublon(request, id):
    matiere = get_object_or_404(Matiere.objects.filter(filiere__annee__ecole=request.ecole), pk=id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Matière créée avec succès.")
        if "conf_annul" in request.POST:
            messages.success(request, "Annulation de la création de la matière")
            matiere.delete()

        return redirect("gestion:frappe:matieres_liste")

    message = "Cette matière a un nom proche d'une matière déjà existante. Voulez-vous la créer quand même?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Créer"))
    return confirmation(request, message, boutons)


@permission_needed("gerer_frappe")
def matieres_editer(request, id):
    matiere = get_object_or_404(Matiere.objects.filter(filiere__annee__ecole=request.ecole), pk=id)
    if request.method == "POST":
        form = MatiereForm(request.POST, instance=matiere, ecole=request.ecole)
        if form.is_valid():
            form.save()
            messages.success(request, "Matière modifiée avec succès.")
            return redirect("gestion:frappe:matieres_liste")
    else:
        form = MatiereForm(instance=matiere, ecole=request.ecole)

    c = {
        "matiere": matiere,
        "form": form,
    }
    return render(request, "gestion/frappe/matieres_editer.html", c)


@permission_needed("gerer_frappe")
def matieres_supprimer(request, id):
    matiere = get_object_or_404(Matiere.objects.filter(filiere__annee__ecole=request.ecole), pk=id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Matière supprimée avec succès.")
            matiere.delete()

        return redirect("gestion:frappe:matieres_liste")

    message = f"Voulez-vous vraiment supprimer la matière « {matiere.nom} » ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)


@permission_needed("gerer_frappe")
def tags_liste(request):
    c = {"tags": Tag.objects.filter(ecole=request.ecole).order_by("nom")}
    return render(request, "gestion/frappe/tags_liste.html", c)


@permission_needed("gerer_frappe")
def tags_nouveau(request):
    if request.method == "POST":
        form = TagForm(request.POST)
        if form.is_valid():
            tag = form.save(commit=False)
            tag.ecole = request.ecole
            tag.save()
            tags_proches = [
                x
                for x in Tag.objects.exclude(id=tag.id).filter(ecole=tag.ecole)
                if (dameraulevenshtein(tag.nom, x.nom) < 3)
            ]
            if len(tags_proches) > 0:
                return redirect("gestion:frappe:tags_nouveau_doublon", id=tag.id)
            else:
                messages.success(request, "Tag créé avec succès.")
                return redirect("gestion:frappe:tags_liste")
    else:
        form = TagForm()

    return render(request, "gestion/frappe/tags_nouveau.html", {"form": form})


@permission_needed("gerer_frappe")
def tags_nouveau_doublon(request, id):
    tag = get_object_or_404(Tag.objects.all(), pk=id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Tag créé avec succès.")
        if "conf_annul" in request.POST:
            messages.success(request, "Annulation de la création du Tag")
            tag.delete()

        return redirect("gestion:frappe:tags_liste")

    message = "Ce tag a un nom proche d'une tag déjà existant. Voulez-vous le créer quand même?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Créer"))
    return confirmation(request, message, boutons)


@permission_needed("gerer_frappe")
def tags_editer(request, id):
    tag = get_object_or_404(Tag.objects.filter(ecole=request.ecole), pk=id)
    if request.method == "POST":
        form = TagForm(request.POST, instance=tag)
        if form.is_valid():
            form.save()
            messages.success(request, "Tag modifié avec succès.")
            return redirect("gestion:frappe:tags_liste")
    else:
        form = TagForm(instance=tag)

    c = {
        "tag": tag,
        "form": form,
    }
    return render(request, "gestion/frappe/tags_editer.html", c)


@permission_needed("gerer_frappe")
def tags_supprimer(request, id):
    tag = get_object_or_404(Tag.objects.filter(ecole=request.ecole), pk=id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Tag supprimé avec succès.")
            tag.delete()

        return redirect("gestion:frappe:tags_liste")

    message = f"Voulez-vous vraiment supprimer le tag « {tag.nom} » ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)


@method_decorator(permission_needed("gerer_frappe"), name="dispatch")
class DocumentsListView(SingleTableView):
    model = Document
    table_class = DocumentTable
    template_name = "gestion/frappe/documents_liste.html"


@permission_needed("gerer_frappe")
def documents_nouveau(request):
    if request.method == "POST":
        form = CreateDocumentForm(request.POST, request.FILES, ecole=request.ecole)
        if form.is_valid():
            document = form.save(commit=False)
            document.auteur = request.user
            document.save()
            form.save_m2m()
            DocumentFichier(document=document, fichier=form.cleaned_data["fichier"], ordre=0).save()
            messages.success(request, "Document créé avec succès.")
            return redirect("gestion:frappe:documents_liste")
    else:
        if not Matiere.objects.filter(filiere__annee__ecole=request.ecole).exists():
            messages.error(request, "Veuillez créer une matière.")
            return redirect("gestion:frappe:documents_liste")

        form = CreateDocumentForm(ecole=request.ecole)

    return render(request, "gestion/frappe/documents_nouveau.html", {"form": form})


def _deplace_fichier_haut(document, id_fichier):
    docfichier = get_object_or_404(DocumentFichier, document=document, pk=id_fichier)

    # on recupère le fichier placé au dessus
    docfichiers = docfichier.document.fichiers().filter(ordre__lt=docfichier.ordre).order_by("-ordre")

    if docfichiers.count() > 0:
        docfichier_upper = docfichiers[0]
        # on inverse les ordres
        docfichier.ordre, docfichier_upper.ordre = (
            docfichier_upper.ordre,
            docfichier.ordre,
        )
        docfichier.save()
        docfichier_upper.save()


def _deplace_fichier_bas(document, id_fichier):
    docfichier = get_object_or_404(DocumentFichier, document=document, pk=id_fichier)

    # on recupère le fichier placé en dessous
    docfichiers = docfichier.document.fichiers().filter(ordre__gt=docfichier.ordre).order_by("ordre")

    if docfichiers.count() > 0:
        docfichier_upper = docfichiers[0]
        # on inverse les ordres
        docfichier.ordre, docfichier_upper.ordre = (
            docfichier_upper.ordre,
            docfichier.ordre,
        )
        docfichier.save()
        docfichier_upper.save()


@permission_needed("gerer_frappe")
def documents_editer(request, id):
    document = get_object_or_404(Document.objects.filter(matiere__filiere__annee__ecole=request.ecole), pk=id)
    if request.method == "POST":
        move_up_id = request.POST.get("move_up")
        if move_up_id is not None:
            _deplace_fichier_haut(document, move_up_id)
            return redirect("gestion:frappe:documents_editer", id=id)

        move_down_id = request.POST.get("move_down")
        if move_down_id is not None:
            _deplace_fichier_bas(document, move_down_id)
            return redirect("gestion:frappe:documents_editer", id=id)

        form = EditDocumentForm(request.POST, request.FILES, instance=document, ecole=request.ecole)
        if form.is_valid():
            form.save()

            if form.cleaned_data["fichier"]:
                # on va le placer à la fin, on recupère l'ordre maximum
                ordre = document.fichiers().aggregate(Max("ordre"))["ordre__max"] + 1
                DocumentFichier(document=document, fichier=form.cleaned_data["fichier"], ordre=ordre).save()

            messages.success(request, "Document modifié avec succès.")
            return redirect("gestion:frappe:documents_liste")
    else:
        form = EditDocumentForm(instance=document, ecole=request.ecole)

    c = {
        "document": document,
        "form": form,
    }
    return render(request, "gestion/frappe/documents_editer.html", c)


@permission_needed("gerer_frappe")
def documents_supprimer(request, id):
    document = get_object_or_404(Document.objects.filter(matiere__filiere__annee__ecole=request.ecole), pk=id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Document supprimé avec succès.")
            document.delete()

        return redirect("gestion:frappe:documents_liste")

    message = f"Voulez-vous vraiment supprimer le document « {document.nom} » ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)


@permission_needed("gerer_frappe")
def docfichiers_supprimer(request, id):
    docfichier = get_object_or_404(
        DocumentFichier.objects.filter(document__matiere__filiere__annee__ecole=request.ecole),
        pk=id,
    )

    if docfichier.document.fichiers().count() == 1:
        messages.error(request, "Vous devriez plutôt supprimer le document entier.")
        return redirect("gestion:frappe:documents_editer", docfichier.document.pk)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Fichier supprimé avec succès.")
            docfichier.delete()

        return redirect("gestion:frappe:documents_editer", docfichier.document.pk)

    message = f"Voulez-vous vraiment supprimer le fichier « {docfichier} » ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)


@permission_needed("gerer_frappe")
def commentaires_liste(request):
    commentaires = (
        Commentaire.objects.filter(document__matiere__filiere__annee__ecole=request.ecole)
        .order_by("-derniere_modif")
        .select_related("document")
    )
    return render(
        request,
        "gestion/frappe/commentaires_liste.html",
        {"commentaires": commentaires},
    )


@permission_needed("gerer_frappe")
def commentaires_editer(request, id):
    commentaire = get_object_or_404(
        Commentaire.objects.filter(document__matiere__filiere__annee__ecole=request.ecole),
        pk=id,
    )

    if request.method == "POST":
        form = CommentaireForm(request.POST, instance=commentaire)
        if form.is_valid():
            form.save()
            messages.success(request, "Commentaire modifié avec succès.")
            return redirect("gestion:frappe:commentaires_liste")
    else:
        form = CommentaireForm(instance=commentaire)

    c = {
        "commentaire": commentaire,
        "form": form,
    }
    return render(request, "gestion/frappe/commentaires_editer.html", c)


@permission_needed("gerer_frappe")
def commentaires_supprimer(request, id):
    commentaire = get_object_or_404(
        Commentaire.objects.filter(document__matiere__filiere__annee__ecole=request.ecole),
        pk=id,
    )

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Commentaire supprimé avec succès.")
            commentaire.delete()

        return redirect("gestion:frappe:commentaires_liste")

    message = f"Voulez-vous vraiment supprimer le commentaire de {commentaire.auteur.get_full_name()} sur le document « {commentaire.document.nom} » ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)
